<?php

/*
   Relativistic Linear Motion Type 1

   SCENARIO:
   Object accelerates from rest to distance D light units at a given
   G-factor.

   This program computes the home time and proper (local) time that
   the moving object takes to reach the target distance and the
   final relativistic speed reached at distance D, taking into
   account the effect of relativistic time-dilation as well.

   AUTHOR  : Jay Tanner
   LICENSE : Public Domain

*/

   ob_start();

// Define the program cookie name and set it to expire in 30 days.
// This cookie can be shared by other programs in the same common
// home folder together.

   $CookieName = 'Relativistic_Accelerated_Linear_Motion_Type_1';
   $SetToExpireIn30Days = time() + 30*86400;

// Define script path and filename.
   $_AUTHOR_          = 'Jay Tanner';
   $_PROGRAM_VERSION_ = ''; $at = "&#97;&#116;"; $UTC = "&#85;&#84;&#67";
   $_SCRIPT_PATH_     = Filter_Input(INPUT_SERVER, 'SCRIPT_FILENAME');
   $_RUN_             = Filter_Input(INPUT_POST, 'SCRIPT_NAME');

// Define internal document page title, HTML page heading text and revision date
   $_INTERNAL_TITLE_  = "Relativistic Accelerated Linear Motion Type 1";
   $_INTERFACE_TITLE_ = "<span style='font-size:140%;'>Relativistic Accelerated Linear Motion Type 1</span><br><span style='font-size:100%;'>Simple Linear Acceleration From Rest to Any Given Distance in Space<br><span style='font-size:90%'>PHP Program by Jay Tanner</span>";
   $_REVISION_DATE_   = $_PROGRAM_VERSION_ .'Revised: '. GMDate("l, F d, Y $at h:i:s A   $UTC", FileMTime($_SCRIPT_PATH_));

// Define main TextArea text, background colors
// and HTML row span.
   $TxColor = 'black';
   $BgColor = 'white';
   $ColSpan = 7;

// Text to display while computing.
   $_COMPUTING_ = "TextArea1.innerHTML='         C.O.M.P.U.T.I.N.G --- This may take a few seconds ...';";


// Do this only if the [COMPUTE] button was clicked.
   $w = Filter_Input(INPUT_POST, 'ComputeButton');
   if (!IsSet($w))
     {

// ----------------------------------------------------
// If [COMPUTE] button was clicked and an active cookie
// exists, then restore the previous interface settings
// from it.

       $w = Filter_Input(INPUT_COOKIE, $CookieName);
            if (IsSet($w))
      {
       $CookieDataString = Filter_Input(INPUT_COOKIE, $CookieName);
        list
       (
        $DistLTU,
        $GFactor,
        $TUSymbol,
       ) = Preg_Split("[\|]", $CookieDataString);

      }

   else

// Set the initial default interface startup values.

 {
  $TUSymbol = 'Y';
  $DistLTU  = '1.0';
  $GFactor  = '1.0';

// Store interface settings in cookie.
   $CookieDataString = "$DistLTU|$GFactor|$TUSymbol";
   SetCookie ($CookieName, $CookieDataString, $SetToExpireIn30Days);
  }
      } // End of  if (!IsSet(...))







// =======================================
// Read values of all interface arguments.
// Empty values are set to zero by default.

   $w = Filter_Input(INPUT_POST, 'ComputeButton');
   if (isset($w))
{
   $TUSymbol = trim(Filter_Input(INPUT_POST, 'TUSymbol'));
   if ($TUSymbol == '') {$TUSymbol = 'Y';}
   $TUSymbol = StrToUpper($TUSymbol);

   $DistLTU = trim(Filter_Input(INPUT_POST, 'DistLTU'));
   if ($DistLTU == '') {$DistLTU = '0';}

   $GFactor = trim(Filter_Input(INPUT_POST, 'GFactor'));
   if ($GFactor == '') {$GFactor = '1';}

// ------------------------------------
// Store interface arguments in cookie.

   $CookieDataString = "$DistLTU|$GFactor|$TUSymbol";
   SetCookie ($CookieName, $CookieDataString, $SetToExpireIn30Days);
}




// ========================================
// Check for error. FALSE = Error detected.
// Change this code to detect errors.
//
// Initially, any non-numeric argument causes
// an error.

   $ErrFlag = TRUE;

   if (!Is_Numeric($DistLTU))
      {
       $TxColor = 'white';
       $BgColor = '#CC0000';
       $ErrFlag = FALSE;
       $ErrMssg =
"Invalid Distance Value:  '$DistLTU'

The input must be a numeric value.
";
      }




   if (!Is_Numeric($GFactor) or $GFactor == 0)
     {
      $TxColor = 'white';
      $BgColor = '#CC0000';
      $ErrFlag = FALSE;
      $ErrMssg =
"Invalid G Factor Value:  '$GFactor'

The G Factor must be a numeric value > 0   (1.0 = Earth)
";
     }

// exit("$TUSymbol");

// Check for valid time units symbol.
// Valid = S,M,H,D,W or Y
   $z = StrPos('SMHDWY', $TUSymbol);
   if ($z === FALSE)
      {
       $TxColor = 'white';
       $BgColor = '#CC0000';
       $ErrFlag = FALSE;

       $ErrMssg =
"Invalid time units symbol:  '$TUSymbol'

Time units symbol must be one of the letters
S, M, H, D, W or Y

Where:
S = Seconds
M = Minutes
H = Hours
D = Days
W = Weeks
Y = Years
";
      }

$TU = substr('SMHDWY', $z, 1);









// ================================
// Check if error (FALSE) detected.

if ($ErrFlag === FALSE)
 {
  $TxColor = 'white';
  $BgColor = '#CC0000';
  $TextArea2Text = '';

  $TextArea1Text =
"===== ERROR =====

$ErrMssg";
 }

else

// ===================================================
// Begin computations here if no error detected above.

  {

// Compute statistics assuming all arguments are valid.
   $Type1Stats = Type_1_Stats ($TUSymbol, $DistLTU, $GFactor);

// Extract statistics into their respective working variables.
   list ($D,$G,$T,$TU,$beta,$t, $xt) = PReg_Split("[ ]", $Type1Stats);
   $TU = StrToUpper($TU);

// Construct time units name strings.
   $TUStr = '???';
   if ($TU == 'S') {$TUStr = 'second';}
   if ($TU == 'M') {$TUStr = 'minute';}
   if ($TU == 'H') {$TUStr = 'hour';}
   if ($TU == 'D') {$TUStr = 'day';}
   if ($TU == 'W') {$TUStr = 'week';}
   if ($TU == 'Y') {$TUStr = 'year';}

// Compute Earth minus ship time.
   $TimeDiff = bcSub($T, $t, 32);

// English corrections.
   $Ds  = ($D <> 1)? 's':'';
   $Ts  = ($T <> 1)? 's':'';
   $ts  = ($t <> 1)? 's':'';
   $Tts = ($TimeDiff > 1)? 's':'';
   $s   = 's';

// Compute alternate time values.
   $TDays = T_Units_to_Days ($T, $TU);
   $TTimesStr = Str_Replace('.000s', 's', Days_to_WDHMS ($TDays));
   $tDays = T_Units_to_Days ($t, $TU);
   $tTimesStr = Str_Replace('.000s', 's', Days_to_WDHMS ($tDays));

   $TtDiff = T_Units_to_Days ($TimeDiff, $TU);
   $TtDiffStr = Str_Replace('.000s', 's', Days_to_WDHMS ($TtDiff));

// Compute alternate speed values in km/h and mi/h
   $vKmSec = bcMul($beta, '299792.458', 32);
   $vKmHr  = bcMul($vKmSec, '3600', 16);
   $vMiHr  = bcDiv($vKmHr, '1.609344', 16);

// Special caps adjustment.
   $xTUStr = UCFirst($TUStr);

// Format Light Time and G factor.
   $DistLTU = bcAdd($DistLTU, '0', 16);
   $GFactor = bcAdd($GFactor, '0', 16);
   $DistLTU = RTrim(RTrim($DistLTU,'0'),'.');
   $GFactor = RTrim(RTrim($GFactor,'0'),'.');











// Construct output text area 1 block.
   $TextArea1Text =
"---------
SCENARIO: A body accelerating along a straight line in space.

Starting from rest, the body accelerates at a given G factor until
reaching distance D, at which point all acceleration stops and the
body has reached the final relativistic coasting speed &beta;.

This program computes the time interval differences for both the
moving body and the home clock due to the effect of relativistic
time-dilation during the accelerated motion and the final speed
achieved at the end.

=====================================
UNITS, TARGET DISTANCE D AND G FACTOR

For Time Units  =  $xTUStr$s
and Dist Units  =  Light $xTUStr$s

D = $D Light $xTUStr$Ds
and
G = $G (1.0 = Earth)

-------------------------------------------------------------------
Home clock time taken to accelerate to distance D light $TUStr$Ts

T = $T $TUStr$Ts
  = $TTimesStr

-------------------------------------------------------------------
Body clock time taken to accelerate to distance D light $TUStr$Ts

t = $t $TUStr$ts
  = $tTimesStr

-------------------------------------------------------------------
Time difference (Home Clock &minus; Body Clock)

T &minus; t = $TimeDiff $TUStr$Tts
      = $TtDiffStr

-------------------------------------------------------------------
Final relativistic speed at endpoint distance D light $TUStr$Ts

&beta; = v/c = $beta

  = $vKmHr km/h
  = $vMiHr mi/h

When the acceleration stops, this is the final constant speed
at which the body will then coast through space forever.

";
  }



// TextArea2.
   $TextArea2Text =
"
This program helps to study the effect of relativistic time-
dilation at different linear acceleration rates and distances.

The body can be a spacecraft or any body in accelerated linear
motion through space. When the body represents a spacecraft,
then times in years and distances in light years (default)
may be the most practical units to apply.

The home clock refers to a clock in the home (lab) rest frame.
The body clock refers to a clock that travels with the body.

According to relativity, the relatively moving body will measure
less clock time than the relatively static home clock during the
motion.
";






// Determine the number of text rows to use in the output TextAreas.
   $TextArea1Rows = 2 + Substr_Count($TextArea1Text, "\n");
   $TextArea2Rows = 2 + Substr_Count($TextArea2Text, "\n");



print <<< _HTML

<!DOCTYPE HTML>
<HTML lang='en'>

<head>

<title>Linear Relativistic Acceleration Type 1</title>

<meta name="viewport" content="width=device-width, initial-scale=1">
<meta http-equiv="content-type" content="text/html; charset=UTF-8">
<meta name="description" content="Linear Relativistic Acceleration Type 1 - Simple acceleration from rest with relativistic time-dilation.">
<meta name="keywords" content="relativity,relativistic motion,relativistic linear motion,relativistic linear acceleration,">
<meta name="author" content="Jay Tanner - https://www.PHPScienceLabs.com">
<meta http-equiv="pragma"  content="no-cache">
<meta http-equiv="expires" content="-1">
<meta name="robots"    content="index,follow">
<meta name="googlebot" content="index,follow">

<link rel="canonical" href="https://www.phpsciencelabs.com/relativistic-accelerated-linear-motion-type-1/">


<style type='text/css'>
 BODY
{color:silver; background-color:black;  font-family:Verdana; font-size:100%}

 IMG {border-radius:8px;}

 TABLE
{font-size:100%; border: 1px solid black;}

 TD
{
 color:black; background-color:white; line-height:150%; font-size:100%;
 padding:6px; text-align:center;
}


 PRE
{
 color:black; background-color:white; font-family:monospace; font-size:150%;
 font-weight:bold; text-align:left; padding:4px;
}

 CODE
{font-family:monospace; font-size:150%; font-weight:normal;}

 TEXTAREA
{
 color:black; background:white; font-family:monospace; font-size:150%;
 font-weight:bold; border-radius: 8px; border:1px solid black; padding:4px;
 white-space:pre; letter-spacing:1px;
}

 INPUT[type="text"]::-ms-clear {width:0; height:0;}

 INPUT[type="text"]
{
 color:black; background:white; font-size:150%; font-family:monospace;
 font-weight:bold; text-align:center; box-shadow:2px 2px 3px #666666;
 border:2px solid black; border-radius:4px;
}

 INPUT[type="text"]:focus
{
 background:white; box-shadow:2px 2px 3px #666666; font-family:monospace;
 font-size:150%; border:2px solid blue; text-align:center; font-weight:bold;
}

 INPUT[type="submit"]
{font-weight:bold;}

 INPUT[type="submit"]:hover
{box-shadow:2px 2px 5px #555555; font-weight:bold;}

 HR {background:red; height:4px; border:0px;}

 A:link
{
 font-size:100%; background:transparent; color:blue; font-family:Verdana;
 font-weight:bold; text-decoration:none; line-height:175%; padding:3px;
 border:1px solid transparent;
}

 A:visited {font-size:100%; background:transparent; color:blue;}

 A:hover
{
 font-size:100%; background:yellow; color:black; border:1px solid black;
 box-shadow:1px 1px 3px #222222;
}

 A:active  {font-size:100%; background:yellow; color:black;}

::selection      {background-color:yellow; color:black;}
::-moz-selection {background-color:yellow; color:black;}
</style>

</head>

<body>

<form name='form1' method='post' action="$_RUN_">

<table width='820' border='0' cellspacing='1' cellpadding='3'>

<tr><td colspan="33%" style='color:white; background-color:#000066; border:2px solid white; border-radius:8px 8px 0px 0px;'>$_INTERFACE_TITLE_</td></tr>


<tr>
<td colspan="1">Time&nbsp;Units&nbsp;Symbol<br><input name='TUSymbol'  type='text' value="$TUSymbol"  size='2'  maxlength='1' title=' S=Sec,  M=Min,  H=Hours,  D=Days,  W=Weeks,  Y=Years '></td>

<td colspan="1">Distance&nbsp;in&nbsp;Light&nbsp;Time&nbsp;Units<br><input name='DistLTU'  type='text' value="$DistLTU"  size='25' maxlength='24'></td>

<td colspan="1">Acceleration&nbsp;G&minus;Factor&nbsp;(1.0&nbsp;=&nbsp;Earth)<br><input name='GFactor'  type='text' value="$GFactor"  size='25' maxlength='24'></td>
</tr>

<tr><td colspan="$ColSpan" style='background-color:black;'>
<input name='ComputeButton' type='submit' value=' C O M P U T E ' OnClick="$_COMPUTING_">
</td></tr>



<tr>
<td colspan="$ColSpan" style='background-color:black;'><br>
<span style='color:white;'>Double-Click Within Text Area to Select ALL Text</span><br>
<textarea name='TextArea1' style='color:$TxColor; background:$BgColor; padding:6px; border:2px solid white;' cols="70" rows="$TextArea1Rows" ReadOnly OnDblClick='this.select();' OnMouseUp='return true;'>
$TextArea1Text
</textarea></td>
</tr>



<tr>
<td colspan="$ColSpan" style='background:black;'>
<textarea name='TextArea2' style='color:black; background:white; padding:6px;' cols="70" rows="$TextArea2Rows" readonly>
$TextArea2Text
</textarea></td>
</tr>

<tr><td colspan='3' style='border-radius:8px;'>
<div style='font-size:85%; color:black; background-color:LightYellow; text-align:center; border:1px solid black; border-radius:5px; box-shadow:1px 1px 3px #222222;'><b>To generally apply the following relativistic equations, distances must be reckoned in related light-time<br>units according to the
simple rules given below.&nbsp; This allows for simplified relativistic motion equations<br>expressed in terms of basic space-time geometry.</b></div>

<br>
The distance units and the value of <i><b>S</b></i> used depends on the choice of time units used in the computations.<br>

<ul style='text-align:left;'>
<li><b>IN THE COMPUTATIONS</b>:<br></li>
<li>c&nbsp; = &nbsp;Speed of Light&nbsp; = &nbsp;299792458 m/s<br></li>
<li>g&nbsp; = &nbsp;Standard gravitational acceleration rate on Earth;&nbsp; = &nbsp;9.80665 m/s&#178;&nbsp; = &nbsp;G factor 1.0</li>
<li>a&nbsp; = &nbsp;General acceleration rate in m/s;&nbsp; = &nbsp; g &#215; (G factor)</li>
<li>G&nbsp; = &nbsp;G factor&nbsp; = &nbsp;a/g<br></li>
<li>1 AU = 149597870700 m<br></li>
<li>1 Standard (Julian) Year&nbsp; = &nbsp;31557600 seconds;&nbsp; = &nbsp;365.25 days<hr><br></li>
<li>If time is to be reckoned in seconds, then distances must be reckoned in light seconds.<br><i><b>S</b></i>&nbsp; = &nbsp;1&nbsp; = &nbsp;Seconds per second<br>1 Light Second&nbsp; = &nbsp;299792458 m&nbsp; = &nbsp;0.002003988804100004 AU<br><br></li>
<li>If time is to be reckoned in minutes, then distances must be reckoned in light minutes.<br><i><b>S</b></i>&nbsp; = &nbsp;60&nbsp; = &nbsp;Seconds per minute<br>1 Light Minute&nbsp; = &nbsp;17987547480 m&nbsp; = &nbsp;0.1202393282460002 AU<br><br></li>
<li>If time is to be reckoned in hours, then distances must be reckoned in light hours.<br><i><b>S</b></i>&nbsp; = &nbsp;3600&nbsp; = &nbsp;Seconds per hour<br>1 Light Hour&nbsp; = &nbsp;1079252848800 m&nbsp; = &nbsp;7.2143596947600137 AU<br><br></li>
<li>If time is to be reckoned in days, then distances must be reckoned in light days.<br><i><b>S</b></i>&nbsp; = &nbsp;86400&nbsp; = &nbsp;Seconds per day<br>1 Light Day&nbsp; = &nbsp;25902068371200 m&nbsp; = &nbsp;173.1446326742403293 AU<br><br></li>
<li>If time is to be reckoned in weeks, then distances must be reckoned in light weeks.<br><i><b>S</b></i>&nbsp; = &nbsp;604800&nbsp; = &nbsp;Seconds per week<br>1 Light Week&nbsp; = &nbsp;181314478598400 m&nbsp; = &nbsp;1212.0124287196823049 AU<br><br></li>
<li>If time is to be reckoned in years, then distances must be reckoned in light years (default).<br><i><b>S</b></i>&nbsp; = &nbsp;31577600&nbsp; = &nbsp;Seconds per standard (Julian) year of 365.25 days<br>1 Light Year&nbsp; = &nbsp;9466726321740800 m&nbsp; = &nbsp;63281.1568603482803449 AU<br><br><hr></li>
</ul>

</div>
<br>
</td></tr>

<tr><td colspan='3' style='text-align:center; background:black;'><img src="images/relativistic-linear-motion-type-1.png"></td></tr>

<tr><td colspan="$ColSpan" style='color:gray; background:black;'>Program by $_AUTHOR_<br>$_REVISION_DATE_</td></tr>


</table>
</form>

<br><br><br><br><br><br><br><br><br><br><br><br>
<br><br><br><br><br><br><br><br><br><br><br><br>
</body>

</HTML>



_HTML;




/*
   ------------------------------------------------------------------------
   This function computes the numerical statistics for relativistic linear
   motion type 1.

   ARGUMENTS:
   TU = Time Units Symbol (NOT case sensitive)
        'S' = Seconds
        'M' = Minutes
        'H' = Hours
        'D' = Days
        'W' = Weeks
        'Y' = Years

   DistLU  = Distance to accelerate in light-time units

   GFactor = Acceleration experienced aboard ship


   DEPENDENCIES:
   None

   ERRORS:
   On error FALSE is returned if any given argument is invalid.
   ------------------------------------------------------------------------
*/

   function Type_1_Stats ($TU='Y', $DistLU=1, $GFactor=1)
{
   $D = trim($DistLU);
   $G = trim($GFactor);
   $U = substr(StrToUpper($TU),0,1);

// Error if either D or G-factor are non-numeric values.
   if (!Is_Numeric($D) or !Is_Numeric($G)) {return FALSE;}

   $Q = 48; // Internal working decimals
   $q = 32; // Returned decimals limit

   $a = bcMul($G, '9.80665', $Q);

// Set time units FACTOR according to units symbol.
   $S = '';
   if ($U == 'S') {$S = '1';}
   if ($U == 'M') {$S = '60';}
   if ($U == 'H') {$S = '3600';}
   if ($U == 'D') {$S = '86400';}
   if ($U == 'W') {$S = '604800';}
   if ($U == 'Y') {$S = '31557600';}

// Error if invalid time units symbol.
   if ($S == '') {return FALSE;}

// Relativistic acceleration potential
// for distance (D) in light units.
   $A = bcDiv(bcMul($a, $S, $Q),'299792458', $Q);

// T = SqRt(D� + 2*D/A)
   $T = bcSqRt(bcAdd(bcMul($D,$D,$Q),bcDiv(bcMul('2',$D,$Q),$A,$Q),$Q),$Q);

// X = D + 1/A
   $X = bcAdd($D, bcDiv('1', $A, $Q), $Q);

// Relativistic speed at distance (D).
// b = T/X
   $b = bcDiv($T, $X, $q);

// Compute proper time (t) in double-precision.
// k = A*(X+T)
// t = ln(k) / A
   $XplusT = bcAdd($X, $T, $Q);
   $k = bcMul($A, $XplusT, $Q);
   $Logk = bcAdd(BC_Ln($k, $q), '0', $q);
   $t = bcDiv(bcAdd($Logk, '0', $q), $A, $q);

// Double-precision (t) value.
   $xt = bcDiv(bcAdd(BC_Ln($k, $q), '0', $q), $A, 16);


// Format (T, b) to (q) decimals.
   $T = bcAdd($T, '0', $q);
   $b = bcAdd($b, '0', $q);

   return "$D $G $T $U $b $t $xt";
}







/*
  ============================================================================
  This function computes the arbitrary-precision
  natural logarithm for any positive (x) argument
  to up to 100 decimals.

  SPECIAL CASE:
  If (x) is a decimal value (0 < x < 1), then Ln(x) = -Ln(1/x)

  ERRORS:
  Returns error message if non-numeric or if (x <= 0).

  NO DEPENDENCIES
  ============================================================================
*/


   function BC_Ln ($xArg, $Decimals=16)
{
   $x = trim($xArg);
   if (StrPos($x, '-') !== FALSE or $x <= 0 or !Is_Numeric($x))
  {return "ERROR: BC_Ln()\n\nArgument must be numeric and <= 0";}

   $q = trim($Decimals);

   if ($q > 100) {$q = 100;}
   if ($q <=  0) {$q =   1;}

   $Q = abs(floor($q)) + 10;

// Handle case of fractional (x) values where (0 < x < 1).
   $sign = ''; if ($x > 0 and $x < 1) {$sign = '-'; $x = bcDiv('1', $x, $Q);}

// Natural logarithm of 10 to 110 decimals (used as internal conversion constant).
   $LN_10 = '2.30258509299404568401799145468436420760110148862877297603332790096757260967735248023599720508959829834196778403';

// Determine power of 10 in positive (x) and
// where to place the decimal in the result.
   $dpos = 1;
   if($x >= 10)
     {
      $dpos = floor(log10($x) + 1);
      $x = Str_Replace('.', '', $x);
      $x = substr($x,0,1).'.'.substr($x,1, StrLen($x));
     }

   $PrecisionLimit = '0.'.Str_Repeat('0', $q).'1';
   $n = $term = 1;
   $sum = 0;
   $k = bcDiv(bcSub($x, '1', $Q), $x, $Q);

   while (abs($term) > $PrecisionLimit)
  {
   $numer = bcpow($k, $n, $Q);
   $denom = $n;
   $term  = bcDiv($numer, $denom, $Q);
   $sum   = bcAdd($sum, $term, $Q);
   $n++;
  }
   $sum = bcAdd($sum, bcMul($dpos-1, $LN_10, $Q), $Q);

// round the sum to the given number of decimals. DONE
   return $sign.bcAdd($sum, Str_Replace('1','5', $PrecisionLimit), $q);

} // end of  BC_Ln(...)





/*
   ===========================================================================
   This function converts decimal days into a 'weeks, days, hours, minutes, seconds string in the format 'wk dd hh mm ss.sss'

   ARGUMENTS:
   Days       = Decimal days value as a numerical string, like '123.456789'
   ssDecimals = Decimals used for the seconds part.

   EXAMPLES:
   Arguments ('184.3657209')      returns '26wk 2d 08h 46m 38s'
   Arguments ('184.3657209', 1)   returns '26wk 2d 08h 46m 38.3s'
   Arguments ('184.3657209', 3)   returns '26wk 2d 08h 46m 38.285s'

   ERRORS:
   An error message is returned if days argument is non-numeric.

   NO DEPENDENCIES
   ===========================================================================
*/

   function Days_to_WDHMS ($Days, $ssDecimals=0)
{
// Read days argument.  Error if non-numeric string.
   $days  = trim("$Days");  if (!Is_Numeric($days))
   {return "ERROR: Days_to_WDHMS()\n\nArgument must be a numeric value.";}

// Set decimals for seconds part.
   $decimals = floor(trim($ssDecimals)); // 01.234
   $w = 3 + $decimals;
   $ssFormatStr = "%0$w.$decimals" . 'f';
   if ($decimals == 0) {$ssFormatStr = '%02d.000';}

// Read and preserve any negative numerical sign and
// work with absolute values internally. Any negative
// sign will be reattached to the returned output.
   $sign  = ($days < 0)? '-':'';
   $days  = Str_Replace('+','', Str_Replace('-','', $days));

// define the letter (d) as the symbol for days.
   $d = 'd';

// Compute weeks
   $weeks = floor($days / 7);
   $wk = ($weeks <> 1)? 'wk ':'wk ';

// Compute residual days.
   if ($weeks == 0) {$weeks = $wk = '';}
   else
      {$days = $days - 7*$weeks;}

// Use 36 decimals for internal work, but limit
// output to a maximum of 16 decimals.
   $Q = 36;
   $q = abs(IntVal(trim($decimals)));
   if ($q > 16) {$q = 16;}

// Compute whole days.
   $dd = bcAdd('0',$days);

// Compute hours.
   $hours = 24*($days - $dd);
   $hours = bcMul('24', bcSub($days, $dd, $Q),$Q);
   $hh = bcAdd('0', $hours);

// Compute minutes.
   $minutes = bcMul('60', bcSub($hours, $hh, $Q),$Q);
   $mm = bcAdd('0', $minutes);

// Compute seconds.
   $seconds = bcMul('60', bcSub($minutes, $mm, $Q),$Q);

// Do some output pre-formatting of time elements.
   $hh = floor($hh);
   $mm = floor($mm);
   $ss = SPrintF($ssFormatStr, $seconds);

// Account for 60s glitch.
   if ($ss == 60)
  {
   $ss = 0;  $mm += 1;  if ($mm == 60){$hh += 1;}
  }

// Do final output formatting.
   $hh = SPrintF("%02d", $hh).'h';
   $mm = SPrintF("%02d", $mm).'m';
   $ss = SPrintF($ssFormatStr, $ss).'s';

// Done.
   return "$sign$weeks$wk$dd$d $hh $mm $ss";

} // end of  Days_to_WDHMS (...)





   function T_Units_to_Days ($TValueStr, $TUnitsStr)
{
   $T = trim($TValueStr);
   $U = substr(StrToUpper(trim($TUnitsStr)), 0,1);

   if (!Is_Numeric($T) or $U == '') {return FALSE;}

   $q = 32;

// ------------------------------------------------------
// Compute days equivalent to given time value and units.

   $days = '';

   if ($U == 'S') {$days = bcDiv($T, '86400',  $q);}
   if ($U == 'M') {$days = bcDiv($T, '1440',   $q);}
   if ($U == 'H') {$days = bcDiv($T, '24',     $q);}
   if ($U == 'D') {$days = bcDiv($T, '1',      $q);}
   if ($U == 'W') {$days = bcMul($T, '7',      $q);}
   if ($U == 'Y') {$days = bcMul($T, '365.25', $q);}

   return $days;
}

/*
   1609.344 m =
   0.0000053681937522257481207215693197992 light seconds

   1 AU = 499 LS

*/






?>





